﻿//Aufgabenstellung
//
//1.Import
//
//Als Teil des Entwicklungsteams sollen Sie einen Import für neue Nutzer der jeweiligen Kunden entwickeln.
//Die Kunden schicken zu diesem Zweck CSV-Dateien mit den Daten von neuen Nutzern.
//Diese Daten müssen in die bestehenden Datentabellen eingespielt werden.
//Dafür sind folgende Anforderungen formuliert:
//
//	• Ein Sachbearbeiter ruft eine Konsolen-App auf, in welcher er die Datei und die
//	    Kundennummer des Kunden als Parameter übergibt: Die Beispieldatei hat den Namen "import_yutani.csv" und
//	    gehört zum Kunden mit der Nummer "K2-002"
//
//	• Die Datei hat den Aufbau:
//      - Ein Nutzer mit Anrede, Vorname, Nachname und Geburtsdatum
//      - Ggf. Adresse: Straße mit Hausnummer, PLZ, Stadt
//      - Ggf. E-Mail
//      - Ggf. eine oder mehrere Telefonnummern: Vorwahl ohne führende 0 und Nummer
//
//    	• Ein Nutzer darf für einen Kunden nicht mehrmals importiert werden.
//
//    	• Die Daten sollen wie folgt validiert werden:
//    		○ Geburtsdatum: TT.MM.JJJJ
//    		○ Valide E-Mail-Adresse
//    		○ PLZ: exakt 5 Nummern
//    		○ Telefonnummer 3 bis 5 Nummern für die Vorwahl und 4 bis 10 Nummern für die Hauptnummer. Keine Sonderzeichen
//
//	• Nicht valide Datensätze von Nutzern sollen in einem Protokoll erfasst werden, ebenso bereits vorhandene Nutzer.
//	  Der komplette Datensatz eines Nutzers darf dann nicht importiert werden


using System;
using System.IO;
using MySql.Data;
using MySql.Data.MySqlClient;
using MySqlX.XDevAPI;
using System.Data.Common;
using System.Reflection.PortableExecutable;
using System.Xml;

class Program
{
    static void Main(string[] args)
    {
        //Connection zum Server

        MySql.Data.MySqlClient.MySqlConnection myConnection;
        string myConnectionString;
        //set the correct values for your server, user, password and database name
        myConnectionString = "server=localhost;uid=root;pwd=root;database=import_export"; //Hier greife ich auf import_export zu

        try
        {
            myConnection = new MySql.Data.MySqlClient.MySqlConnection(myConnectionString);
            //open a connection
            myConnection.Open();

            // create a MySQL command and set the SQL statement with parameters
            MySqlCommand myCommand = new MySqlCommand();
            myCommand.Connection = myConnection;
//            myCommand.CommandText = @"SELECT * FROM workshop WHERE maxParticipants = @code;";

            //Selber eingefügt ab hier
            myCommand.CommandText = @"SELECT * FROM user"; //Hier tragen wir unsere SELECT ein
            //Hier ende

//            myCommand.Parameters.AddWithValue("@code", "12");

            // execute the command and read the results
            using MySqlDataReader myReader = myCommand.ExecuteReader();

            //Hier dann den CSV Reader benutzen?
            while (myReader.Read())
            {
                int id = myReader.GetInt32("id");
                string name = myReader.GetString("firstname");
                Console.WriteLine(id + " " + name);
            }

            myConnection.Close();
        }
        catch (MySql.Data.MySqlClient.MySqlException ex)
        {

        }

        //Consolen-Ausgabe und Abfrage   Eventuell hoch in das While schieben

        Console.WriteLine("Willkommen");
        Console.Write("Bitte geben Sie nun die Kundenummer ein: ");
        string kundennummer = Console.ReadLine();
        Console.Write("Bitte geben Sie die Datei-Parameter ein: ");
        string dateiBezeichnung = Console.ReadLine();

        string filePath = "path/to/your/" + dateiBezeichnung;    //Wo wird die CSV vor dem einlesen abgelegt? Extra Ordner dafür erstellen

        //Überprüfung, ob die Datei exisitiert mit File.Exists()

        if (!File.Exists(filePath))
        {
            Console.WriteLine("Die Datei-Parameter konnten nicht gefunden werden. Bitte versuche es erneut");
        }


        //Schwierigkeit: Bei User beginnen und dann alle anderen Tabellen durch Foreign Keys verbinden/verweisen
        //Procedure INSERT anschauen - Elegante Lösung
        //Auto Increment beachten

        //Umsetzung mit Arrays: Beginnend mit durchzählen für Länge Array, darauffolgende Unterteilung in Blöcken
        //(CSV Datei einteilen: (Beginnend immer mit Herr/Frau) , dann Erstellung Tabelle User, dann andere Tabellen mit Foreign Keys verknüpfen
    }
}


/*   Das hier wäre eine Lösungsvariante. Diese beruht auf den vorhandenen Daten

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text.RegularExpressions;
using MySql.Data.MySqlClient;

class Program
{
    static void Main(string[] args)
    {
        // Connection zum Server
        string myConnectionString = "server=localhost;uid=root;pwd=root;database=import_export";

        Console.WriteLine("Willkommen");
        Console.Write("Bitte geben Sie nun die Kundennummer ein: ");
        string kundennummer = Console.ReadLine();
        Console.Write("Bitte geben Sie die Datei-Parameter ein: ");
        string dateiBezeichnung = Console.ReadLine();

        string filePath = "path/to/your/" + dateiBezeichnung; // Adjust the path to your CSV file

        if (!File.Exists(filePath))
        {
            Console.WriteLine("Die Datei-Parameter konnten nicht gefunden werden. Bitte versuche es erneut");
            return;
        }

        List<string> invalidEntries = new List<string>();
        List<string> existingEntries = new List<string>();

        // Einlesen der CSV-Datei und Verarbeitung der Daten
        string[] lines = File.ReadAllLines(filePath);
        List<User> users = ParseCsv(lines, invalidEntries);

        using (MySqlConnection myConnection = new MySqlConnection(myConnectionString))
        {
            myConnection.Open();

            foreach (User user in users)
            {
                // Überprüfen, ob der Nutzer bereits existiert
                if (UserExists(myConnection, user, kundennummer))
                {
                    existingEntries.Add($"{user.Firstname} {user.Lastname}, {user.Email}");
                    continue;
                }

                // Nutzer einfügen
                InsertUser(myConnection, user, kundennummer);
            }
        }

        // Ungültige und bereits vorhandene Einträge protokollieren
        LogEntries("invalid_entries.log", invalidEntries);
        LogEntries("existing_entries.log", existingEntries);

        Console.WriteLine("Import abgeschlossen.");
    }

    static List<User> ParseCsv(string[] lines, List<string> invalidEntries)
    {
        List<User> users = new List<User>();
        User currentUser = null;

        foreach (string line in lines)
        {
            string[] parts = line.Split(';');

            if (parts[0] == "Herr" || parts[0] == "Frau")
            {
                if (currentUser != null)
                {
                    users.Add(currentUser);
                }
                currentUser = new User
                {
                    Gender = parts[0],
                    Firstname = parts[1],
                    Lastname = parts[2],
                    Birthdate = parts[3]
                };
                continue;
            }

            if (parts[0] == "Adresse" && currentUser != null)
            {
                currentUser.Address = new Address
                {
                    Street = parts[1],
                    PostalCode = parts[2],
                    City = parts[3]
                };
                continue;
            }

            if (parts[0] == "E-Mail" && currentUser != null)
            {
                currentUser.Email = parts[1];
                if (!IsValidEmail(currentUser.Email))
                {
                    invalidEntries.Add($"Ungültige E-Mail: {currentUser.Email}");
                    currentUser = null;
                }
                continue;
            }

            if (parts[0] == "Telefon" && currentUser != null)
            {
                if (!IsValidPhone(parts[1], parts[2]))
                {
                    invalidEntries.Add($"Ungültige Telefonnummer: {parts[1]} {parts[2]}");
                    currentUser = null;
                }
                else
                {
                    currentUser.Phones.Add(new Phone
                    {
                        Prefix = parts[1],
                        Number = parts[2]
                    });
                }
                continue;
            }
        }

        if (currentUser != null)
        {
            users.Add(currentUser);
        }

        return users;
    }

    static bool IsValidEmail(string email)
    {
        try
        {
            var addr = new System.Net.Mail.MailAddress(email);
            return addr.Address == email;
        }
        catch
        {
            return false;
        }
    }

    static bool IsValidPhone(string prefix, string number)
    {
        return Regex.IsMatch(prefix, @"^\d{3,5}$") && Regex.IsMatch(number, @"^\d{4,10}$");
    }

    static bool UserExists(MySqlConnection connection, User user, string clientId)
    {
        string query = @"SELECT COUNT(*) FROM user WHERE firstname = @firstname AND lastname = @lastname AND clientId = @clientId";
        MySqlCommand cmd = new MySqlCommand(query, connection);
        cmd.Parameters.AddWithValue("@firstname", user.Firstname);
        cmd.Parameters.AddWithValue("@lastname", user.Lastname);
        cmd.Parameters.AddWithValue("@clientId", clientId);
        return Convert.ToInt32(cmd.ExecuteScalar()) > 0;
    }

    static void InsertUser(MySqlConnection connection, User user, string clientId)
    {
        string query = @"INSERT INTO user (clientId, genderId, firstname, lastname, birthdate) VALUES (@clientId, @genderId, @firstname, @lastname, @birthdate)";
        MySqlCommand cmd = new MySqlCommand(query, connection);
        cmd.Parameters.AddWithValue("@clientId", clientId);
        cmd.Parameters.AddWithValue("@genderId", user.Gender == "Herr" ? 1 : (user.Gender == "Frau" ? 2 : 3));
        cmd.Parameters.AddWithValue("@firstname", user.Firstname);
        cmd.Parameters.AddWithValue("@lastname", user.Lastname);
        cmd.Parameters.AddWithValue("@birthdate", DateTime.ParseExact(user.Birthdate, "dd.MM.yyyy", CultureInfo.InvariantCulture));

        cmd.ExecuteNonQuery();
        long userId = cmd.LastInsertedId;

        if (user.Address != null)
        {
            query = @"INSERT INTO address (userId, street, postalcode, city) VALUES (@userId, @street, @postalcode, @city)";
            cmd = new MySqlCommand(query, connection);
            cmd.Parameters.AddWithValue("@userId", userId);
            cmd.Parameters.AddWithValue("@street", user.Address.Street);
            cmd.Parameters.AddWithValue("@postalcode", user.Address.PostalCode);
            cmd.Parameters.AddWithValue("@city", user.Address.City);
            cmd.ExecuteNonQuery();
        }

        if (!string.IsNullOrEmpty(user.Email))
        {
            query = @"INSERT INTO email (userId, email) VALUES (@userId, @Email)";
            cmd = new MySqlCommand(query, connection);
            cmd.Parameters.AddWithValue("@userId", userId);
            cmd.Parameters.AddWithValue("@Email", user.Email);
            cmd.ExecuteNonQuery();
        }

        foreach (var phone in user.Phones)
        {
            query = @"INSERT INTO phone (userId, phoneprefix, phonenumber) VALUES (@userId, @phoneprefix, @phonenumber)";
            cmd = new MySqlCommand(query, connection);
            cmd.Parameters.AddWithValue("@userId", userId);
            cmd.Parameters.AddWithValue("@phoneprefix", phone.Prefix);
            cmd.Parameters.AddWithValue("@phonenumber", phone.Number);
            cmd.ExecuteNonQuery();
        }
    }

    static void LogEntries(string filePath, List<string> entries)
    {
        if (entries.Count > 0)
        {
            File.AppendAllLines(filePath, entries);
        }
    }
}

class User
{
    public string Gender { get; set; }
    public string Firstname { get; set; }
    public string Lastname { get; set; }
    public string Birthdate { get; set; }
    public Address Address { get; set; }
    public string Email { get; set; }
    public List<Phone> Phones { get; set; } = new List<Phone>();
}

class Address
{
    public string Street { get; set; }
    public string PostalCode { get; set; }
    public string City { get; set; }
}

class Phone
{
    public string Prefix { get; set; }
    public string Number { get; set; }
}

*/